=== modified file 'drizzled/db.cc'
--- drizzled/db.cc	2008-11-18 22:12:56 +0000
+++ drizzled/db.cc	2008-11-21 16:06:07 +0000
@@ -1012,7 +1012,7 @@
     }
   }
   if (session->killed ||
-      (tot_list && mysql_rm_table_part2(session, tot_list, 1, 0, 1, 1)))
+      (tot_list && mysql_rm_table_part2(session, tot_list, 1, 0, 1)))
     goto err;
 
   my_dirend(dirp);  

=== modified file 'drizzled/handler.cc'
--- drizzled/handler.cc	2008-11-19 20:56:34 +0000
+++ drizzled/handler.cc	2008-11-21 16:06:07 +0000
@@ -1368,30 +1368,80 @@
 }
 
 
+struct handlerton_delete_table_args {
+  Session *session;
+  const char *path;
+  handler *file;
+  int error;
+};
+
+static bool deletetable_handlerton(Session *unused1 __attribute__((unused)),
+                                   plugin_ref plugin,
+                                   void *args)
+{
+  struct handlerton_delete_table_args *dtargs= (struct handlerton_delete_table_args *) args;
+
+  Session *session= dtargs->session;
+  const char *path= dtargs->path;
+
+  handler *file;
+  char tmp_path[FN_REFLEN];
+
+  if(dtargs->error!=ENOENT) /* already deleted table */
+    return false;
+
+  handlerton *table_type= plugin_data(plugin, handlerton *);
+
+  if(!table_type)
+    return false;
+
+  if(!(table_type->state == SHOW_OPTION_YES && table_type->create))
+    return false;
+
+  if ((file= table_type->create(table_type, NULL, session->mem_root)))
+    file->init();
+  else
+    return false;
+
+  path= check_lowercase_names(file, path, tmp_path);
+  int error= file->ha_delete_table(path);
+
+  if(error!=ENOENT)
+  {
+    dtargs->error= error;
+    if(dtargs->file)
+      delete dtargs->file;
+    dtargs->file= file;
+    return true;
+  }
+
+  return false;
+}
+
 /**
   This should return ENOENT if the file doesn't exists.
   The .frm file will be deleted only if we return 0 or ENOENT
 */
-int ha_delete_table(Session *session, handlerton *table_type, const char *path,
+int ha_delete_table(Session *session, const char *path,
                     const char *db, const char *alias, bool generate_warning)
 {
-  handler *file;
-  char tmp_path[FN_REFLEN];
-  int error;
+  TABLE_SHARE dummy_share;
   Table dummy_table;
-  TABLE_SHARE dummy_share;
+
+  struct handlerton_delete_table_args dtargs;
+  dtargs.error= ENOENT;
+  dtargs.session= session;
+  dtargs.path= path;
+  dtargs.file= NULL;
+
+  plugin_foreach(NULL, deletetable_handlerton, DRIZZLE_STORAGE_ENGINE_PLUGIN,
+                 &dtargs);
 
   memset(&dummy_table, 0, sizeof(dummy_table));
   memset(&dummy_share, 0, sizeof(dummy_share));
   dummy_table.s= &dummy_share;
 
-  /* DB_TYPE_UNKNOWN is used in ALTER Table when renaming only .frm files */
-  if (table_type == NULL ||
-      ! (file=get_new_handler((TABLE_SHARE*)0, session->mem_root, table_type)))
-    return(ENOENT);
-
-  path= check_lowercase_names(file, path, tmp_path);
-  if ((error= file->ha_delete_table(path)) && generate_warning)
+  if (dtargs.error && generate_warning)
   {
     /*
       Because file->print_error() use my_error() to generate the error message
@@ -1410,10 +1460,11 @@
     dummy_share.table_name.length= strlen(alias);
     dummy_table.alias= alias;
 
+    handler *file= dtargs.file;
     file->change_table_ptr(&dummy_table, &dummy_share);
 
     session->push_internal_handler(&ha_delete_table_error_handler);
-    file->print_error(error, 0);
+    file->print_error(dtargs.error, 0);
 
     session->pop_internal_handler();
 
@@ -1421,11 +1472,14 @@
       XXX: should we convert *all* errors to warnings here?
       What if the error is fatal?
     */
-    push_warning(session, DRIZZLE_ERROR::WARN_LEVEL_ERROR, error,
+    push_warning(session, DRIZZLE_ERROR::WARN_LEVEL_ERROR, dtargs.error,
                 ha_delete_table_error_handler.buff);
   }
-  delete file;
-  return(error);
+
+  if(dtargs.file)
+    delete dtargs.file;
+
+  return dtargs.error;
 }
 
 /****************************************************************************

=== modified file 'drizzled/handler.h'
--- drizzled/handler.h	2008-11-18 22:12:56 +0000
+++ drizzled/handler.h	2008-11-21 16:06:07 +0000
@@ -1208,7 +1208,7 @@
                     const char *db, const char *table_name,
                     HA_CREATE_INFO *create_info,
                     bool update_create_info);
-int ha_delete_table(Session *session, handlerton *db_type, const char *path,
+int ha_delete_table(Session *session, const char *path,
                     const char *db, const char *alias, bool generate_warning);
 
 /* statistics and info */

=== modified file 'drizzled/show.cc'
--- drizzled/show.cc	2008-11-19 20:56:34 +0000
+++ drizzled/show.cc	2008-11-21 16:06:07 +0000
@@ -2345,20 +2345,12 @@
   }
   else
   {
-    enum legacy_db_type not_used;
     char path[FN_REFLEN];
     (void) build_table_filename(path, sizeof(path), db_name->str, 
                                 table_name->str, reg_ext, 0);
-    if (mysql_frm_type(session, path, &not_used)) 
-    {
+
       table->field[3]->store(STRING_WITH_LEN("BASE Table"),
                              system_charset_info);
-    }
-    else
-    {
-      table->field[3]->store(STRING_WITH_LEN("ERROR"),
-                             system_charset_info);
-    }
 
     if (session->is_error() && session->main_da.sql_errno() == ER_NO_SUCH_TABLE)
     {

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2008-11-19 20:56:34 +0000
+++ drizzled/sql_table.cc	2008-11-21 16:06:07 +0000
@@ -322,7 +322,7 @@
     LOCK_open during wait_if_global_read_lock(), other threads could not
     close their tables. This would make a pretty deadlock.
   */
-  error= mysql_rm_table_part2(session, tables, if_exists, drop_temporary, 0, 0);
+  error= mysql_rm_table_part2(session, tables, if_exists, drop_temporary, 0);
 
   if (need_start_waiting)
     start_waiting_global_read_lock(session);
@@ -364,8 +364,7 @@
 */
 
 int mysql_rm_table_part2(Session *session, TableList *tables, bool if_exists,
-                         bool drop_temporary, bool drop_view,
-                         bool dont_log_query)
+                         bool drop_temporary, bool dont_log_query)
 {
   TableList *table;
   char path[FN_REFLEN], *alias;
@@ -416,8 +415,6 @@
   {
     char *db=table->db;
     handlerton *table_type;
-    enum legacy_db_type frm_db_type;
-
 
     error= drop_temporary_table(session, table);
 
@@ -485,8 +482,8 @@
                                         FN_IS_TMP : 0);
     }
     if (drop_temporary ||
-        ((table_type == NULL && (access(path, F_OK) && ha_create_table_from_engine(session, db, alias))) ||
-         (!drop_view && mysql_frm_type(session, path, &frm_db_type) != true)))
+        ((table_type == NULL && (access(path, F_OK) && ha_create_table_from_engine(session, db, alias))))
+        )
     {
       // Table was not found on disk and table can't be created from engine
       if (if_exists)
@@ -499,17 +496,12 @@
     else
     {
       char *end;
-      if (table_type == NULL)
-      {
-        mysql_frm_type(session, path, &frm_db_type);
-        table_type= ha_resolve_by_legacy_type(session, frm_db_type);
-      }
       // Remove extension for delete
       *(end= path + path_length - reg_ext_length)= '\0';
-      error= ha_delete_table(session, table_type, path, db, table->table_name,
+      error= ha_delete_table(session, path, db, table->table_name,
                              !dont_log_query);
       if ((error == ENOENT || error == HA_ERR_NO_SUCH_TABLE) && 
-	  (if_exists || table_type == NULL))
+	  if_exists)
       {
 	error= 0;
         session->clear_error();
@@ -631,7 +623,7 @@
     != 0        Error
 */
 
-bool quick_rm_table(handlerton *base,const char *db,
+bool quick_rm_table(handlerton *base __attribute__((unused)),const char *db,
                     const char *table_name, uint32_t flags)
 {
   char path[FN_REFLEN];
@@ -646,7 +638,7 @@
 
   error|= delete_table_proto_file(path);
 
-  return(ha_delete_table(current_session, base, path, db, table_name, 0) ||
+  return(ha_delete_table(current_session, path, db, table_name, 0) ||
               error);
 }
 
@@ -4408,7 +4400,6 @@
   char path[FN_REFLEN];
   ha_rows copied= 0,deleted= 0;
   handlerton *old_db_type, *new_db_type, *save_old_db_type;
-  legacy_db_type table_type;
 
   new_name_buff[0]= '\0';
 
@@ -4461,7 +4452,6 @@
     into the main table list, like open_tables does).
     This code is wrong and will be removed, please do not copy.
   */
-  (void)mysql_frm_type(session, new_name_buff, &table_type);
 
   if (!(table= open_n_lock_single_table(session, table_list, TL_WRITE_ALLOW_READ)))
     return(true);

=== modified file 'drizzled/sql_table.h'
--- drizzled/sql_table.h	2008-11-15 17:33:00 +0000
+++ drizzled/sql_table.h	2008-11-21 16:06:07 +0000
@@ -35,7 +35,7 @@
 bool mysql_rm_table(Session *session,TableList *tables, bool if_exists,
                     bool drop_temporary);
 int mysql_rm_table_part2(Session *session, TableList *tables, bool if_exists,
-                         bool drop_temporary, bool drop_view, bool log_query);
+                         bool drop_temporary, bool log_query);
 bool quick_rm_table(handlerton *base,const char *db,
                     const char *table_name, uint32_t flags);
 void close_cached_table(Session *session, Table *table);

=== modified file 'drizzled/table.cc'
--- drizzled/table.cc	2008-11-19 20:56:34 +0000
+++ drizzled/table.cc	2008-11-21 16:06:07 +0000
@@ -3550,48 +3550,6 @@
   return length;
 }
 
-/*
-  Check type of .frm if we are not going to parse it
-
-  SYNOPSIS
-  mysql_frm_type()
-  path        path to file
-
-  RETURN
-  false       error
-  true       table
-*/
-
-bool mysql_frm_type(Session *, char *path, enum legacy_db_type *dbt)
-{
-  File file;
-  unsigned char header[10];     /* This should be optimized */
-  int error;
-
-  *dbt= DB_TYPE_UNKNOWN;
-
-  if ((file= open(path, O_RDONLY)) < 0)
-    return false;
-  error= my_read(file, (unsigned char*) header, sizeof(header), MYF(MY_NABP));
-  my_close(file, MYF(MY_WME));
-
-  if (error)
-    return false;
-
-  /*  
-    This is just a check for DB_TYPE. We'll return default unknown type
-    if the following test is true (arg #3). This should not have effect
-    on return value from this function (default FRMTYPE_TABLE)
-   */  
-  if (header[0] != (unsigned char) 254 || header[1] != 1 ||
-      (header[2] != FRM_VER && header[2] != FRM_VER+1 &&
-       (header[2] < FRM_VER+3 || header[2] > FRM_VER+4)))
-    return true;
-
-  *dbt= (enum legacy_db_type) (uint) *(header + 3);
-  return true;                   // Is probably a .frm table
-}
-
 /****************************************************************************
  Functions for creating temporary tables.
 ****************************************************************************/

=== modified file 'drizzled/table.h'
--- drizzled/table.h	2008-11-19 20:56:34 +0000
+++ drizzled/table.h	2008-11-21 16:06:07 +0000
@@ -44,9 +44,6 @@
 /*************************************************************************/
 
 
-bool mysql_frm_type(Session *session, char *path, enum legacy_db_type *dbt);
-
-
 class Field_timestamp;
 class Field_blob;
 

=== modified file 'storage/heap/ha_heap.cc'
--- storage/heap/ha_heap.cc	2008-11-19 08:07:10 +0000
+++ storage/heap/ha_heap.cc	2008-11-21 16:06:07 +0000
@@ -569,8 +569,7 @@
 
 int ha_heap::delete_table(const char *name)
 {
-  int error= heap_delete_table(name);
-  return error == ENOENT ? 0 : error;
+  return heap_delete_table(name);
 }
 
 

=== modified file 'storage/innobase/row/row0mysql.c'
--- storage/innobase/row/row0mysql.c	2008-10-28 14:01:55 +0000
+++ storage/innobase/row/row0mysql.c	2008-11-21 16:06:07 +0000
@@ -3042,22 +3042,7 @@
 	table = dict_table_get_low(name);
 
 	if (!table) {
-		err = DB_TABLE_NOT_FOUND;
-		ut_print_timestamp(stderr);
-
-		fputs("  InnoDB: Error: table ", stderr);
-		ut_print_name(stderr, trx, TRUE, name);
-		fputs(" does not exist in the InnoDB internal\n"
-		      "InnoDB: data dictionary though MySQL is"
-		      " trying to drop it.\n"
-		      "InnoDB: Have you copied the .frm file"
-		      " of the table to the\n"
-		      "InnoDB: MySQL database directory"
-		      " from another database?\n"
-		      "InnoDB: You can look for further help from\n"
-		      "InnoDB: http://dev.mysql.com/doc/refman/5.1/en/"
-		      "innodb-troubleshooting.html\n",
-		      stderr);
+		err = ENOENT;
 		goto funct_exit;
 	}
 

=== modified file 'storage/myisam/mi_delete_table.c'
--- storage/myisam/mi_delete_table.c	2008-07-15 20:45:59 +0000
+++ storage/myisam/mi_delete_table.c	2008-11-21 16:06:07 +0000
@@ -24,7 +24,7 @@
   char from[FN_REFLEN];
 
   fn_format(from,name,"",MI_NAME_IEXT,MY_UNPACK_FILENAME|MY_APPEND_EXT);
-  if (my_delete_with_symlink(from, MYF(MY_WME)))
+  if (my_delete_with_symlink(from, 0))
     return(my_errno);
   fn_format(from,name,"",MI_NAME_DEXT,MY_UNPACK_FILENAME|MY_APPEND_EXT);
   return(my_delete_with_symlink(from, MYF(MY_WME)) ? my_errno : 0);

=== modified file 'tests/Makefile.am'
--- tests/Makefile.am	2008-10-20 00:23:44 +0000
+++ tests/Makefile.am	2008-11-21 16:06:07 +0000
@@ -161,6 +161,9 @@
         delete \
         distinct \
 	drizzleslap \
+	drop_table \
+	drop_table_if_exists \
+	drop_table_temp \
 	empty_table \
         endspace \
         flush2 \
@@ -218,7 +221,6 @@
         limit \
         lock_tables_lost_commit \
 	main.myisam-blob \
-	main.myisam-system \
 	mysqldump-compat \
 	mysqldump-max \
 	order_by \

=== modified file 'tests/include/mix1.inc'
--- tests/include/mix1.inc	2008-10-19 19:35:14 +0000
+++ tests/include/mix1.inc	2008-11-21 16:06:07 +0000
@@ -578,21 +578,6 @@
 DROP TABLE t1,t2,t3;
 
 #
-# Test bug when trying to drop data file which no InnoDB directory entry
-#
-
-create table t1 (a int) engine=innodb;
-copy_file $MYSQLTEST_VARDIR/master-data/test/t1.frm $MYSQLTEST_VARDIR/master-data/test/bug29807.frm;
---error 1146
-select * from bug29807;
-drop table t1;
---error 1051
-drop table bug29807;
-create table bug29807 (a int);
-drop table bug29807;
-
-
-#
 # Bug #29154: LOCK TABLES is not atomic when >1 InnoDB tables are locked
 #
 

=== added file 'tests/r/drop_table.result'
--- tests/r/drop_table.result	1970-01-01 00:00:00 +0000
+++ tests/r/drop_table.result	2008-11-21 16:06:07 +0000
@@ -0,0 +1,2 @@
+create table t1 (a int);
+drop table t1;

=== added file 'tests/r/drop_table_if_exists.result'
--- tests/r/drop_table_if_exists.result	1970-01-01 00:00:00 +0000
+++ tests/r/drop_table_if_exists.result	2008-11-21 16:06:07 +0000
@@ -0,0 +1,6 @@
+drop table if exists t1;
+Warnings:
+Note	1051	Unknown table 't1'
+drop table if exists t2;
+Warnings:
+Note	1051	Unknown table 't2'

=== added file 'tests/r/drop_table_temp.result'
--- tests/r/drop_table_temp.result	1970-01-01 00:00:00 +0000
+++ tests/r/drop_table_temp.result	2008-11-21 16:06:07 +0000
@@ -0,0 +1,4 @@
+create table t1 (a int);
+create temporary table t1 (a int);
+drop table t1;
+drop table t1;

=== modified file 'tests/r/innodb_mysql.result'
--- tests/r/innodb_mysql.result	2008-10-19 19:35:14 +0000
+++ tests/r/innodb_mysql.result	2008-11-21 16:06:07 +0000
@@ -528,14 +528,6 @@
 SELECT * FROM t3 WHERE a = 'uk';
 a
 DROP TABLE t1,t2,t3;
-create table t1 (a int) engine=innodb;
-select * from bug29807;
-ERROR 42S02: Table 'test.bug29807' doesn't exist
-drop table t1;
-drop table bug29807;
-ERROR 42S02: Unknown table 'bug29807'
-create table bug29807 (a int);
-drop table bug29807;
 CREATE TABLE t1 (a INT) ENGINE=InnoDB;
 CREATE TABLE t2 (a INT) ENGINE=InnoDB;
 switch to connection c1

=== removed file 'tests/r/myisam-system.result'
--- tests/r/myisam-system.result	2008-06-25 05:29:13 +0000
+++ tests/r/myisam-system.result	1970-01-01 00:00:00 +0000
@@ -1,13 +0,0 @@
-drop table if exists t1,t2;
-create table t1 (a int) engine=myisam;
-drop table if exists t1;
-Warnings:
-Error	2	Can't find file: 't1' (errno: 2)
-create table t1 (a int) engine=myisam;
-drop table t1;
-Got one of the listed errors
-create table t1 (a int) engine=myisam;
-drop table t1;
-Got one of the listed errors
-drop table t1;
-ERROR 42S02: Unknown table 't1'

=== added file 'tests/t/drop_table.test'
--- tests/t/drop_table.test	1970-01-01 00:00:00 +0000
+++ tests/t/drop_table.test	2008-11-21 16:06:07 +0000
@@ -0,0 +1,2 @@
+create table t1 (a int);
+drop table t1;
\ No newline at end of file

=== added file 'tests/t/drop_table_if_exists.test'
--- tests/t/drop_table_if_exists.test	1970-01-01 00:00:00 +0000
+++ tests/t/drop_table_if_exists.test	2008-11-21 16:06:07 +0000
@@ -0,0 +1,2 @@
+drop table if exists t1;
+drop table if exists t2;
\ No newline at end of file

=== added file 'tests/t/drop_table_temp.test'
--- tests/t/drop_table_temp.test	1970-01-01 00:00:00 +0000
+++ tests/t/drop_table_temp.test	2008-11-21 16:06:07 +0000
@@ -0,0 +1,4 @@
+create table t1 (a int);
+create temporary table t1 (a int);
+drop table t1;
+drop table t1;
\ No newline at end of file

=== removed file 'tests/t/myisam-system.test'
--- tests/t/myisam-system.test	2008-06-25 05:29:13 +0000
+++ tests/t/myisam-system.test	1970-01-01 00:00:00 +0000
@@ -1,21 +0,0 @@
-#
-# Test how DROP TABLE works if the index or data file doesn't exists
-
-# Initialise
---disable_warnings
-drop table if exists t1,t2;
---enable_warnings
-
-create table t1 (a int) engine=myisam;
---remove_file $MYSQLTEST_VARDIR/master-data/test/t1.MYI
-drop table if exists t1;
-create table t1 (a int) engine=myisam;
---remove_file $MYSQLTEST_VARDIR/master-data/test/t1.MYI
---error 1051,6
-drop table t1;
-create table t1 (a int) engine=myisam;
---remove_file $MYSQLTEST_VARDIR/master-data/test/t1.MYD
---error 1105,6,29
-drop table t1;
---error 1051
-drop table t1;

